package ru.dbotthepony.mc.otm.capability;

import net.minecraftforge.energy.IEnergyStorage;

import javax.annotation.Nonnull;
import java.math.BigDecimal;

// IEnergyStorage for direct compat with Forge Energy
public interface IMatteryEnergyStorage extends IEnergyStorage {
	// such as cables. This is something that would work only with energy storage
	@Nonnull
	BigDecimal extractEnergyOuter(BigDecimal howMuch, boolean simulate);

	// for internal needs, e.g. for work
	// CAN also be used by something that does evil
	// e.g. sucking out energy anomaly should use this
	@Nonnull
	BigDecimal extractEnergyInner(BigDecimal howMuch, boolean simulate);

	// energy is received from outside, e.g. cables
	@Nonnull
	BigDecimal receiveEnergyOuter(BigDecimal howMuch, boolean simulate);

	// energy is received from inside, e.g. generator generates power
	@Nonnull
	BigDecimal receiveEnergyInner(BigDecimal howMuch, boolean simulate);

	@Nonnull
	default BigDecimal extractEnergyOuter(long howMuch, boolean simulate) {
		return extractEnergyOuter(new BigDecimal(howMuch, MatteryCapability.ROUND_RULES), simulate);
	}

	@Nonnull
	default BigDecimal extractEnergyOuter(int howMuch, boolean simulate) {
		return extractEnergyOuter(new BigDecimal(howMuch, MatteryCapability.ROUND_RULES), simulate);
	}

	@Nonnull
	default BigDecimal receiveEnergyOuter(long howMuch, boolean simulate) {
		return receiveEnergyOuter(new BigDecimal(howMuch, MatteryCapability.ROUND_RULES), simulate);
	}

	@Nonnull
	default BigDecimal receiveEnergyOuter(int howMuch, boolean simulate) {
		return receiveEnergyOuter(new BigDecimal(howMuch, MatteryCapability.ROUND_RULES), simulate);
	}

	@Nonnull
	default BigDecimal extractEnergyInner(long howMuch, boolean simulate) {
		return receiveEnergyOuter(new BigDecimal(howMuch, MatteryCapability.ROUND_RULES), simulate);
	}

	@Nonnull
	default BigDecimal extractEnergyInner(int howMuch, boolean simulate) {
		return receiveEnergyOuter(new BigDecimal(howMuch, MatteryCapability.ROUND_RULES), simulate);
	}

	@Nonnull
	default BigDecimal receiveEnergyInner(long howMuch, boolean simulate) {
		return receiveEnergyOuter(new BigDecimal(howMuch, MatteryCapability.ROUND_RULES), simulate);
	}

	@Nonnull
	default BigDecimal receiveEnergyInner(int howMuch, boolean simulate) {
		return receiveEnergyOuter(new BigDecimal(howMuch, MatteryCapability.ROUND_RULES), simulate);
	}

	@Nonnull
	BigDecimal getBatteryLevel();

	@Nonnull
	BigDecimal getMaxBatteryLevel();

	default BigDecimal getMissingPower() {
		return getMaxBatteryLevel().subtract(getBatteryLevel(), MatteryCapability.ROUND_RULES);
	}

	@Override
	default int receiveEnergy(int maxReceive, boolean simulate) {
		BigDecimal toReceive = new BigDecimal(maxReceive, MatteryCapability.ROUND_RULES);
		int received = receiveEnergyOuter(toReceive, true).intValue();

		if (received == 0) {
			// Receiving only a fraction
			return 0;
		}

		return receiveEnergyOuter(new BigDecimal(received, MatteryCapability.ROUND_RULES), simulate).intValue();
	}

	@Override
	default int extractEnergy(int maxReceive, boolean simulate) {
		BigDecimal toReceive = new BigDecimal(maxReceive, MatteryCapability.ROUND_RULES);
		int extracted = extractEnergyOuter(toReceive, true).intValue();

		if (extracted == 0) {
			// Extracting only a fraction
			return 0;
		}

		return extractEnergyOuter(new BigDecimal(extracted, MatteryCapability.ROUND_RULES), simulate).intValue();
	}

	@Override
	default int getEnergyStored() {
		BigDecimal level = getBatteryLevel();

		if (level.compareTo(MatteryCapability.INT_MAX_VALUE) == -1) {
			return level.intValue();
		}

		return Integer.MAX_VALUE;
	};

	@Override
	default int getMaxEnergyStored() {
		BigDecimal level = getMaxBatteryLevel();

		if (level.compareTo(MatteryCapability.INT_MAX_VALUE) == -1) {
			return level.intValue();
		}

		return Integer.MAX_VALUE;
	};

	@Override
	default boolean canExtract() {
		return extractEnergyOuter(BigDecimal.ONE, true).compareTo(BigDecimal.ZERO) > 0;
	};

	@Override
	default boolean canReceive() {
		return receiveEnergyOuter(BigDecimal.ONE, true).compareTo(BigDecimal.ZERO) > 0;
	};

	default boolean canExtractOuter() {
		return extractEnergyOuter(BigDecimal.ONE, true).compareTo(BigDecimal.ZERO) > 0;
	};

	default boolean canReceiveOuter() {
		return receiveEnergyOuter(BigDecimal.ONE, true).compareTo(BigDecimal.ZERO) > 0;
	};

	default boolean canExtractInner() {
		return extractEnergyInner(BigDecimal.ONE, true).compareTo(BigDecimal.ZERO) > 0;
	};

	default boolean canReceiveInner() {
		return receiveEnergyInner(BigDecimal.ONE, true).compareTo(BigDecimal.ZERO) > 0;
	};
}
