package ru.dbotthepony.mc.otm.menu;

import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.Slot;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityAndroidStation;
import ru.dbotthepony.mc.otm.capability.IAndroidCapability;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.slot.BatterySlot;

import javax.annotation.Nullable;

public class AndroidStationMenu extends PoweredMachineMenu {
	public static class AndroidStationContainer extends SimpleContainer {
		public final Player player;
		private final IAndroidCapability android;

		public AndroidStationContainer(Player ply) {
			super(1);
			player = ply;

			LazyOptional<IAndroidCapability> capability = ply.getCapability(MatteryCapability.ANDROID);
			android = capability.isPresent() ? capability.resolve().get() : null;

			if (android != null) {
				setItem(0, android.getBatteryItemStack());
			}
		}
	}

	public static class AndroidBatterySlot extends BatterySlot {
		private final IAndroidCapability android;

		public AndroidBatterySlot(AndroidStationContainer container, int index, int x, int y) {
			super(container, index, x, y);
			android = container.android;
		}

		@Override
		public void set(ItemStack stack) {
			super.set(stack);
			android.setBatteryItemStack(stack);
		}
	}

	// Clientside
	public AndroidStationMenu(int containerID, Inventory inventory) {
		this(containerID, inventory,null);
	}

	private final int battery_slot_index;

	// Serverside
	public AndroidStationMenu(
			int containerID,
			Inventory inventory,
			@Nullable BlockEntityAndroidStation tile
	) {
		super(
				Registry.Menus.ANDROID_STATION,
				containerID,
				inventory,
				tile);

		battery_slot_index = addSlot(new AndroidBatterySlot(new AndroidStationContainer(inventory.player), 0, 39, 15)).index;
		addBatterySlot();

		addInventorySlots();
	}

	@Override
	public ItemStack quickMoveStack(Player ply, int slot_index) {
		ItemStack moved = ItemStack.EMPTY;
		Slot get_slot = this.slots.get(slot_index);

		if (get_slot.hasItem()) {
			ItemStack slot_item = get_slot.getItem();
			moved = slot_item.copy();

			if (slot_index < inventory_slot_index_start) {
				// Moving FROM machine TO inventory

				if (!moveItemStackTo(slot_item, inventory_slot_index_start, inventory_slot_index_end + 1, false)) {
					return ItemStack.EMPTY;
				}
			} else if (!moveItemStackTo(slot_item, battery_slot_index, battery_slot_index + 1, true)) {
				// Moving FROM inventory TO machine
				return ItemStack.EMPTY;
			}

			if (slot_item.isEmpty()) {
				get_slot.set(ItemStack.EMPTY);
			} else {
				get_slot.setChanged();
			}
		}

		return moved;
	}
}
