package ru.dbotthepony.mc.otm.menu;

import net.minecraft.core.BlockPos;
import net.minecraft.world.Container;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.inventory.MenuType;
import net.minecraft.world.inventory.Slot;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMatteryPoweredMachine;
import ru.dbotthepony.mc.otm.capability.IMatteryEnergyStorage;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.data.BigDecimalDataContainer;
import ru.dbotthepony.mc.otm.menu.slot.BatterySlot;

import javax.annotation.Nullable;

abstract public class PoweredMachineMenu extends AbstractContainerMenu {
	protected BlockEntityMatteryPoweredMachine tile;
	protected Inventory inventory;
	protected Player ply;
	public BigDecimalDataContainer energy;
	public BigDecimalDataContainer max_energy;

	protected PoweredMachineMenu(
			@Nullable MenuType<?> menuType,
			int containerID,
			Inventory inventory,
			@Nullable BlockEntityMatteryPoweredMachine tile
	) {
		super(menuType, containerID);
		this.tile = tile;
		this.inventory = inventory;
		this.ply = inventory.player;

		if (tile == null) {
			energy = new BigDecimalDataContainer();
			max_energy = new BigDecimalDataContainer();
		} else {
			energy = new BigDecimalDataContainer() {
				@Override
				protected void updateValue() {
					LazyOptional<IMatteryEnergyStorage> capability = tile.getCapability(MatteryCapability.ENERGY);

					if (capability.isPresent())
						setDecimal(capability.resolve().get().getBatteryLevel());
				}
			};

			max_energy = new BigDecimalDataContainer() {
				@Override
				protected void updateValue() {
					LazyOptional<IMatteryEnergyStorage> capability = tile.getCapability(MatteryCapability.ENERGY);

					if (capability.isPresent())
						setDecimal(capability.resolve().get().getMaxBatteryLevel());
				}
			};
		}

		this.addDataSlots(energy);
		this.addDataSlots(max_energy);
	}

	protected void addBatterySlot() {
		addBatterySlot(8, 66);
	}

	protected void addBatterySlot(int x, int y) {
		Container battery_container = tile != null ? tile.battery_container : new SimpleContainer(1);
		this.addSlot(new BatterySlot(battery_container, 0, x, y));
	}

	protected void addInventorySlots() {
		addInventorySlots(97);
	}

	protected int inventory_slot_index_start;
	protected int inventory_slot_index_end;

	protected void addInventorySlots(int offset) {
		boolean first = true;

		for (int i = 0; i < 3; ++i) {
			for (int j = 0; j < 9; ++j) {
				Slot slot = this.addSlot(new Slot(inventory, j + i * 9 + 9, 8 + j * 18, offset + i * 18));

				if (first) {
					first = false;
					inventory_slot_index_start = slot.index;
				}
			}
		}

		Slot last = null;

		for (int k = 0; k < 9; ++k) {
			last = this.addSlot(new Slot(inventory, k, 8 + k * 18, offset + 58));
		}

		inventory_slot_index_end = last.index;
	}

	@Override
	public boolean stillValid(Player player) {
		if (tile == null)
			return true;

		if (player.level.getBlockEntity(tile.getBlockPos()) != tile) {
			return false;
		}

		BlockPos pos = tile.getBlockPos();
		return player.distanceToSqr((double)pos.getX() + 0.5D, (double)pos.getY() + 0.5D, (double)pos.getZ() + 0.5D) <= 64.0D;
	}

	// This method receive Player interactor and slot_index where Shift + Right click occurred
	// It shall return item stack that got moved
	@Override
	public ItemStack quickMoveStack(Player ply, int slot_index) {
		// this.moveItemStackTo(ItemStack, int start_slot_index, int end_slot_index, boolean iteration_order)
		// returns boolean, telling whenever ItemStack was modified (moved or sharnk)
		// false means nothing happened
		// Last boolean determine order of slot iteration, where:
		// if TRUE, iteration order is end_slot_index -> start_slot_index
		// if FALSE iteration order is start_slot_index -> end_slot_index

		return ItemStack.EMPTY;
	}
}
